//
//  CS52_5_4132_1230718_AshleyZeldin
//
//  Ashley Zeldin
//  CS 52 Section 4132
//  October 19, 2015
//
//  BaseballTeam.cpp
//

#include <iostream>
#include <string>
#include "BaseballTeam.h"

namespace CS52
{
    //Operators
    
    //Overload << operator to describe type LEAGUE.
    std::ostream & operator << (std::ostream & outs, BaseballTeam::LEAGUE league)
    {
        switch (league)
        {
            case (BaseballTeam::National): outs << "National"; break;
            case (BaseballTeam::American): outs << "American"; break;
            default: outs << ""; break;
        }
        
        return outs;
    }

    //Overload << operator to describe type DIVISION.
    std::ostream & operator << (std::ostream & outs, BaseballTeam::DIVISION division)
    {
        switch (division)
        {
            case (BaseballTeam::West): outs << "West"; break;
            case (BaseballTeam::Central): outs << "Central"; break;
            case (BaseballTeam::East): outs << "East"; break;
            default: outs << ""; break;
        }
        
        return outs;
    }
    
    //Arrays
    
    //A four-dimensional array of teams. Oy.
    std::string BaseballTeam::allTeams [2][3][5][2] =
    {
        {
            {
                //National League, West Division
                {"Arizona", "Diamondbacks"},
                {"Colorado", "Rockies"},
                {"Los Angeles", "Dodgers"},
                {"San Diego", "Padres"},
                {"San Francisco", "Giants"},
            },
            {
                //National League, Central Division
                {"Chicago", "Cubs"},
                {"Cincinnati", "Reds"},
                {"Milwaukee", "Brewers"},
                {"Pittsburgh", "Pirates"},
                {"St Louis", "Cardinals"},
            },
            {
                //National League, East Division
                {"Atlanta", "Braves"},
                {"Miami", "Marlins"},
                {"New York", "Mets"},
                {"Philadelphia", "Phillies"},
                {"Washington", "Nationals"},
            }
        },
        {
            {
                //American League, West Division
                {"Houston", "Astros"},
                {"Los Angeles", "Angels"},
                {"Oakland", "Athletics"},
                {"Seattle", "Mariners"},
                {"Texas", "Rangers"},
            },
            {
                //American League, Central Division
                {"Chicago", "White Sox"},
                {"Cleveland", "Indians"},
                {"Detroit", "Tigers"},
                {"Kansas City", "Royals"},
                {"Minnesota", "Twins"},
            },
            {
                //American League, East Division
                {"Baltimore", "Orioles"},
                {"Boston", "Red Sox"},
                {"New York", "Yankees"},
                {"Tampa Bay", "Rays"},
                {"Toronto", "Blue Jays"},
            }
        }
    };
    
    //A constant integer variable to represent total number of teams.
    const int BaseballTeam::numTeams = 30;
    
    //An array of teams to handle wins and losses.
    BaseballTeam BaseballTeam::teamWAndL[numTeams];
    
    //Create a list of all teams with name, city, league and division affiliation for parsing.
    void BaseballTeam::createTeamList (int currentSeason)
    {
        //Start with 0 teams.
        int numTeams = 0;
        
        //Iterate over number of leagues, 2.
        for (int i = 0; i < BaseballTeam::numLeagues; i++)
        {
            //Iterate over number of divisions, 3.
            for (int j = 0; j < BaseballTeam::numDivisions; j++)
            {
                //Iterate over number of teams in each division in each league, 5.
                for (int k = 0; k < 5; k++)
                {
                    //List team, constructed with name, city, year of season, league and division.
                    teamWAndL[numTeams] = BaseballTeam(allTeams[i][j][k][1], allTeams[i][j][k][0], currentSeason, (LEAGUE)i, (DIVISION)j);
                    
                    //Increment to move to next index in array.
                    numTeams++;
                }
            }
        }
    }
    
    
    //Constructors

    //Create random baseball team for indecisive fans.
    BaseballTeam::BaseballTeam()
    {
        //Use random number generator to select a team.
        int randomLeague = random() % numLeagues;
        int randomDivision = random() % numDivisions;
        int randomTeam = random() % 5;
        
        //Set team city and name, as well as league and division.
        teamCity = allTeams[randomLeague][randomDivision][randomTeam][0];
        teamName = allTeams[randomLeague][randomDivision][randomTeam][1];
        teamLeague = (LEAGUE)randomLeague;
        teamDivision = (DIVISION)randomDivision;
        
        //Initialize to false 'til team assigned as favorite in main.
        favoriteTeam = false;
        
        //Initialize all stats to 0.
        teamHomeWins = 0;
        teamHomeLosses = 0;
        teamAwayWins = 0;
        teamAwayLosses = 0;
    }
    
    //Create baseball team with default 2015 current season and National League affiliation.
    BaseballTeam::BaseballTeam(std::string name, std::string city, int season, LEAGUE league, DIVISION division)
    {
        //Set team attributes based on passed-in parameters.
        teamName = name;
        teamCity = city;
        currentSeason = season;
        teamLeague = league;
        teamDivision = division;
        
        //Initialize to false 'til team assigned as favorite in main.
        favoriteTeam = false;
        
        //Initialize all stats to 0.
        teamHomeWins = 0;
        teamHomeLosses = 0;
        teamAwayWins = 0;
        teamAwayLosses = 0;
    }
    

//Setters
    
    //Set year of current season.
    void BaseballTeam::setSeason(int season)
    {
        currentSeason = season;
    }
    
    //Set whether team is favorite.
    void BaseballTeam::setFavoriteTeam(bool isFavorite)
    {
        favoriteTeam = isFavorite;
    }
    
    
//Getters
    
    //Which team.
    std::string BaseballTeam::getName() const
    {
        return teamName;
    }
    
    //Which city.
    std::string BaseballTeam::getCity() const
    {
        return teamCity;
    }
    
    //Which season.
    int BaseballTeam::getSeason() const
    {
        return currentSeason;
    }
    
    //Whether this team is the favorite team.
    bool BaseballTeam::isFavoriteTeam() const
    {
        return favoriteTeam;
    }
    
    //Whether affiliated with National League.
    bool BaseballTeam::isNationalLeague() const
    {
        return teamLeague == National;
    }
    
    //Whether affiliated with American League.
    bool BaseballTeam::isAmericanLeague() const
    {
        return teamLeague == American;
    }
    
    //Which division team is affiliated with, West, Central or East.
    BaseballTeam::DIVISION BaseballTeam::whichDivision() const
    {
        return teamDivision;
    }
    
    //Number of home wins.
    int BaseballTeam::homeWins() const
    {
        return teamHomeWins;
    }
    
    //Number of home losses.
    int BaseballTeam::homeLosses() const
    {
        return teamHomeLosses;
    }
    
    //Number of away wins.
    int BaseballTeam::awayWins() const
    {
        return teamAwayWins;
    }
    
    //Number of away losses.
    int BaseballTeam::awayLosses() const
    {
        return teamAwayLosses;
    }
    
    
//Modifiers
    
    //Add win(s) to total, with default parameter of home win.
    void BaseballTeam::win (int count, bool home)
    {
        //This should never happen 'cause it's already handled in main.
        if (count < 0 || count + teamHomeWins + teamHomeLosses + teamAwayWins + teamAwayLosses > 162)
        {
            std::cout << "The Major League Baseball season is 162 games, barring a strike! You can't win that many games!" << std::endl;
        }
        //After playing a game, increment depending on home or away.
        else if (home)
        {
            teamHomeWins += count;
        }
        else
        {
            teamAwayWins += count;
        }
    }
    
    //Add loss(es) to total, with default parameter of home loss.
    void BaseballTeam::lose (int count, bool home)
    {
        //This should never happen 'cause it's already handled in main.
        if (count < 0 || count + teamHomeWins + teamHomeLosses + teamAwayWins + teamAwayLosses > 162)
        {
            std::cout << "The Major League Baseball season is 162 games, barring a strike! You can't lose that many games!" << std::endl;
        }
        //After playing a game, increment depending on home or away.
        else if (home)
        {
            teamHomeLosses += count;
        }
        else
        {
            teamAwayLosses += count;
        }
    }
    
    //Start new season.
    void BaseballTeam::newSeason(int season)
    {
        currentSeason = season;
        
        //This should never happen since we use system time to determine base year.
        if (currentSeason < 1869)
        {
            std::cout << "Major League Baseball wasn't founded 'til 1869! You can't play that season!" << std::endl;
            
            return;
        }
        
        //If favorite team selected.
        if (favoriteTeam)
        {
            std::cout << std::endl << "Welcome to the start of the " << currentSeason << " season! Cheer on your " << teamCity << " " << teamName << " in the " << teamLeague << " League " << teamDivision << "!" << std::endl << std::endl;
        }
        else
        {
            //std::cout << "Welcome to the start of the " << currentSeason << " baseball season! Why don't you (c)hoose your favorite team?" << std::endl;
        }
        
        //Initialize all stats to 0.
        teamHomeWins = 0;
        teamHomeLosses = 0;
        teamAwayWins = 0;
        teamAwayLosses = 0;
    }
    
    //Display win-loss stats.
    void BaseballTeam::printTeam() const
    {
        //Variables to handle total wins and losses.
        int totalWins = teamHomeWins + teamAwayWins;
        int totalLosses = teamHomeLosses + teamAwayLosses;
        int totalGames = totalWins + totalLosses;
        
        //Variables to handle league and division rankings.
        int leagueTeamsAhead = 0;
        int divisionTeamsAhead = 0;
        int allTeamsAhead = 0;
        
        int divisionSecondPlace = 0;
        
        //Parse list of teams.
        for (int i = 0; i < numTeams; i++)
        {
            //If opponent.
            if (teamCity != teamWAndL[i].getCity() || teamName != teamWAndL[i].getName())
            {
                //Add up wins.
                int compareWins = teamWAndL[i].homeWins() + teamWAndL[i].awayWins();
                
                //Compare wins.
                if (totalWins < compareWins)
                {
                    //Differentiate between leagues.
                    bool sameLeague = false;
                    
                    //Count all teams ahead.
                    allTeamsAhead++;

                    //If both teams in American League.
                    if (teamWAndL[i].isAmericanLeague() && teamLeague == American)
                    {
                        //Increment teams ahead in league, and note same league.
                        leagueTeamsAhead++;
                        sameLeague = true;
                    }
                    //If both teams in National League.
                    else if (teamWAndL[i].isNationalLeague() && teamLeague == National)
                    {
                        //Increment teams ahead in league, and note same league.
                        leagueTeamsAhead++;
                        sameLeague = true;
                    }
                    
                    //If in same league and division.
                    if (sameLeague && teamWAndL[i].whichDivision() == teamDivision)
                    {
                        //Increment teams ahead in division.
                        divisionTeamsAhead++;
                        
                        //Track game differential v team at index.
                        int losingByGames = compareWins - totalWins;
                        
                        //If greater than current value, replace.
                        if (losingByGames > divisionSecondPlace)
                        {
                            //Count number of games if in second place in division, for wilcard purposes.
                            divisionSecondPlace = losingByGames;
                        }
                    }
                }
            }
        }
        
        //Start of season, nothing to output except 0-0.
        if (totalGames <= 0)
        {
            std::cout << "The " << currentSeason << " season has just begun! No games played!" << std::endl;
        }
        //Early in season output.
        else if (totalGames < 54)
        {
            std::cout << "Early in the " << currentSeason << " season, ";
            
            if (favoriteTeam)
            {
                std::cout << "your ";
            }
            else
            {
                std::cout << "the ";
            }
            
            std::cout << teamCity << " " << teamName;
            
            //Excellent start.
            if (leagueTeamsAhead == 0)
            {
                std::cout << " are off to a fast start, atop the " << teamLeague << " League out of the " << teamDivision << " division with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
            }
            else
            {
                //Great start.
                if (divisionTeamsAhead == 0)
                {
                    std::cout << " are off to a fast start, atop the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Good start.
                else if (divisionTeamsAhead == 1)
                {
                    std::cout << " are in good position, " << divisionSecondPlace << " games behind the leaders in the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Mediocre start.
                else if (divisionTeamsAhead > 1 && divisionTeamsAhead < 4)
                {
                    std::cout << " have had a so-so showing in the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Bad start.
                else
                {
                    std::cout << " have got off to a dismal start, at the bottom of the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
            }
        }
        //Midseason output.
        else if (totalGames < 162)
        {
            std::cout << "Midway through the " << currentSeason << " season, ";
            
            if (favoriteTeam)
            {
                std::cout << "your ";
            }
            else
            {
                std::cout << "the ";
            }
            
            std::cout << teamCity << " " << teamName;
 
            //Excellent midseason.
            if (leagueTeamsAhead == 0)
            {
                std::cout << " look to be a lock for the postseason, atop the " << teamLeague << " League out of the " << teamDivision << " division with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
            }
            //Great midseason.
            else
            {
                if (divisionTeamsAhead == 0)
                {
                    std::cout << " seem to be well on their way to the postseason atop the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Good midseason.
                else if (divisionTeamsAhead == 1)
                {
                    std::cout << " are " << divisionSecondPlace << " games behind the leaders in the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Mediocre midseason.
                else if (divisionTeamsAhead > 1 && divisionTeamsAhead < 4)
                {
                    std::cout << " are in the middle of a tough " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Bad midseason.
                else
                {
                    std::cout << " sit in the " << teamLeague << " League " << teamDivision << " cellar with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
            }
        }
        else
        {
            //End of season output.
            std::cout << "At the end of the " << currentSeason << " season, ";
            
            if (favoriteTeam)
            {
                std::cout << "your ";
            }
            else
            {
                std::cout << "the ";
            }
            
            std::cout << teamCity << " " << teamName;
            
            //Best in baseball.
            if (allTeamsAhead == 0)
            {
                std::cout << " are the best team in baseball headed into the postseason, atop the " << teamLeague << " League out of the " << teamDivision << " division with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
            }
            //Best in league.
            else if (leagueTeamsAhead == 0)
            {
                std::cout << " are the best team from the " << teamLeague << " League headed into the postseason, atop the " << teamDivision << " division with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
            }
            else
            {
                //Best in division.
                if (divisionTeamsAhead == 0)
                {
                    std::cout << " have clinched the " << teamLeague << " League " << teamDivision << " postseason berth with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Second-best in division, ranked 4 or 5 in league.
                else if (divisionTeamsAhead == 1 && (leagueTeamsAhead == 3 || leagueTeamsAhead == 4))
                {
                    std::cout << " are headed to the wildcard play-in game out of the " << teamLeague << " League " << teamDivision << " with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
                //Not-so-good.
                else
                {
                    std::cout << " will miss the postseason after a disappointing " << teamLeague << " League " << teamDivision << " finish with a " << (teamHomeWins + teamAwayWins) << "-" << (teamHomeLosses + teamAwayLosses) << " record!" << std::endl;
                }
            }

        }
        
        //Records of wins and losses, home and away.
        std::cout << std::endl << "Records: " << std::endl << "Home: " << teamHomeWins << "-" << teamHomeLosses << " | Away: " << teamAwayWins << "-" << teamAwayLosses << std::endl << std::endl;
    }
}