//
//  CS52_7_4132_1230718_AshleyZeldin
//
//  Ashley Zeldin
//  CS 52 Section 4132
//  October 19, 2015
//
//  BaseballTeam.h
//

#ifndef CS52_7_4132_1230718_AshleyZeldin_BaseballTeam_h
#define CS52_7_4132_1230718_AshleyZeldin_BaseballTeam_h

#include <iostream>
#include <string>

namespace CS52
{
    class BaseballTeam
    {
    public:
    
    //Enumerations
        
        //Define divisions.
        enum DIVISION {West, Central, East, numDivisions};
        
        //Define leagues.
        enum LEAGUE {National, American, numLeagues};
        
    //Operators
        
        //Overload << operator to output league.
        friend std::ostream & operator << (std::ostream & outs, LEAGUE league);
        
        //Overload << operator to output division.
        friend std::ostream & operator << (std::ostream & outs, DIVISION division);
        
    //Arrays
        
        //Array of league, division and team info. Declared as static so as not to create an instance of the array every time an instance of the class is constructed.
        static std::string allTeams [2][3][5][2];
        
        //A constant integer variable to represent total number of teams.
        static const int numTeams;

        //Array of teams to handle wins and losses. Declared as static so as not to create an instance of the array every time an instance of the class is constructed.
        static BaseballTeam teamWAndL [30];
        
        //Method that creates list of baseball teams. Declared as static so as not to call method on individual team.
        static void createTeamList (int currentSeason);

        
    //Constructors
        
        //Create generic baseball team.
        BaseballTeam();
        
        //Create baseball team with default 2015 current season and National League affiliation.
        BaseballTeam(std::string name, std::string city, int season = 2015, LEAGUE league = National, DIVISION division = West);
        
    //Setters
        
        //Set year of current season.
        void setSeason(int season);
        
        //Sets favorite team.
        void setFavoriteTeam (bool isFavorite);
        

    //Getters
        
        //Which team.
        std::string getName() const;
        
        //Which city.
        std::string getCity() const;
    
        //Which season.
        int getSeason() const;
        
        //Whether is favorite team.
        bool isFavoriteTeam() const;
        
        //Whether affiliated with National League.
        bool isNationalLeague() const;
        
        //Whether affiliated with American League.
        bool isAmericanLeague() const;
        
        //Which division team is affiliated with, West, Central or East.
        DIVISION whichDivision() const;
        
        //Number of home wins.
        int homeWins() const;
        
        //Number of home losses.
        int homeLosses() const;
        
        //Number of away wins.
        int awayWins() const;
        
        //Number of away losses.
        int awayLosses() const;
        
        
    //Modifiers
        
        //Add win(s) to total, with default parameter of home win.
        void win (int count = 1, bool home = true);
        
        //Add loss(es) to total, with default parameter of home loss.
        void lose (int count = 1, bool home = true);
        
        //Start new season.
        void newSeason(int season);
        
        //Display team stats.
        void printTeam() const;

  
    private:
        
        //Name of team.
        std::string teamName;
        
        //City of team.
        std::string teamCity;
        
        //Tracks league affiliation.
        LEAGUE teamLeague;
        
        //Tracks division affiliation.
        DIVISION teamDivision;

        //Tracks current season.
        int currentSeason;
        
        //Tracks favorite team selection.
        bool favoriteTeam;
        
        //Tracks team wins at home.
        int teamHomeWins;
        
        //Tracks team losses at home.
        int teamHomeLosses;
        
        //Tracks team wins on the road.
        int teamAwayWins;
        
        //Tracks team losses on the road.
        int teamAwayLosses;
        
    };
}

#endif
