//
//  CS52_6_4132_1230718_AshleyZeldin
//
//  Ashley Zeldin
//  CS 52 Section 4132
//  October 19, 2015
//
//  Purpose: To create a baseball season simulator that allows individual game play, entire season simulation and display of scores and records.
//  Input: Program accepts character input for option selection, and input of team city and name for various choices.
//  Output: Program outputs option prompts, displays individual game scores and full season scores, win-loss records and witty commentary.
//  Note: I left out "of Anaheim" in reference to the Angels 'cause 1. it's asinine, 2. I couldn't be fussed.
//

#include <iostream>
#include <string>
#include <time.h>
#include "BaseballTeam.h"

using namespace CS52;

//An enumeration for error handling.
enum ERROR_TYPE
{
    teamNotFound,
    favoriteTeamNotDefined,
    tooManyGamesPlayed,
    seasonNotStarted,
    invalidSelection,
    cantUnplayGame
};

// Declaration of function prototype to check if strings are equal, disregarding case.
bool caseInsensitiveStringCompare(const std::string str1, const std::string str2);
//Declaration of function prototype to check specified team against list of all teams to find a match.
BaseballTeam * specifyTeam(std::string teamIdentifier);
//Declaration of function prototype to select random opponent to play individual game against favorite team.
int randomOpponent(BaseballTeam * favoriteTeam);
//Declaration of function prototype to play individual game, favorite team v random team.
void playGame(BaseballTeam * teamA, BaseballTeam * teamB, bool displayResults);
//Declaration of function prototype to play rigged game, favorite team v random team.
void playRiggedGame(BaseballTeam * teamA, BaseballTeam * teamB, bool favoriteHome, bool favoriteWon);
//Declaration of function prototype to simulate all teams playing a single game, with option to rig a game for manual entry.
void allTeamsPlay(bool displayResults, bool rigGame = false, bool favoriteHome = false, bool favoriteWon = false);
//Declaration of function prototype to simulate all teams playing remainder of season up to 162 games.
void simulateSeason(int gamesToPlay, bool displayResults);
//Declaration of function prototype to handle erroneous input.
void callError(int errorCount, ERROR_TYPE errorType);

int main()
{
    //ASCII art!
    std::string baseballIntro =
    "     F F F F    A A    N     N  T T T T    A A     S S S   Y     Y\n"
    "     F        A     A  N N   N     T     A     A  S        Y     Y\n"
    "     F F F    A A A A  N   N N     T     A A A A   S S S    Y Y Y\n"
    "     F        A     A  N     N     T     A     A        S        Y\n"
    "     F        A     A  N     N     T     A     A   S S S    Y Y Y\n"
    "\n"
    "B B B      A A     S S S   E E E E  B B B      A A    L        L\n"
    "B    B   A     A  S        E        B    B   A     A  L        L\n"
    "B B B    A A A A   S S S   E E E    B B B    A A A A  L        L\n"
    "B     B  A     A        S  E        B     B  A     A  L        L\n"
    "B B B    A     A   S S S   E E E E  B B B    A     A  L L L L  L L L L \n\n";
    
    std::cout << baseballIntro;
    
    std::cout << "Welcome to your baseball team simulator!" << std::endl;
    
    //Seed random number generator.
    srandom(time(NULL));
    
    //Use system time to get current year.
    time_t systemTime;
    time(& systemTime);
    tm * timeStruct = localtime(& systemTime);
    int currentSeason = timeStruct->tm_year + 1900;
    
    //Create list of all teams.
    BaseballTeam::createTeamList(currentSeason);
    
    //Create reference to favorite, which will be filled by an existing team later.
    BaseballTeam * favoriteTeam = NULL;
    
    //Declare and initialize boolean variable to handle infinite repetition of switch statement.
    bool repeater = true;
    
    //Declare and initialize integer variables to handle number of seasons played since program start, number of games played in current season, number of errors encountered.
    int seasonsPlayed = 0;
    int gamesPlayed = 0;
    int errorCount = 0;
    
    //As long as...
    do {
        std::cout << "You may:" << std::endl;
        std::cout << "* (c)hoose a favorite team;" << std::endl << "* start a (n)ew season;" << std::endl << "* (p)lay a game;" << std::endl << "* (m)anually input wins or losses;" << std::endl << "* (s)imulate season;" << std::endl << "* (d)isplay win-loss records;" << std::endl << "* (q)uit." << std::endl;
        
        //Declare and initialize a character variable to handle user input of choice for switch statement.
        char choice = ' ';
        
        std::cout << "Please make a selection now: ";
        std::cin >> choice;
        
        //Flush buffer. A bit more verbose than fflush from C.
        std::cin.clear();
        std::cin.ignore(INT_MAX, '\n');
        
        switch (choice)
        {
            //Choose a favorite team.
            case 'c':
            {
                std::cout << std::endl << "Which team do you root, root, root for? Start by entering the city or team name. You may leave this blank to choose a random team." << std::endl;
                std::string teamIdentifier = "";
                std::getline(std::cin, teamIdentifier);
                
                //If a favorite team already indicated.
                if (favoriteTeam != NULL)
                {
                    //Remove favorite team marker from current favorite, dereferenced by ->.
                    favoriteTeam->setFavoriteTeam(false);
                    
                    //Reset pointer reference.
                    favoriteTeam = NULL;
                }
                
                //If no team selected.
                if (teamIdentifier == "")
                {
                    //Declare and initialize a random team.
                    BaseballTeam randomTeam;
                    randomTeam.setSeason(currentSeason);
                    
                    //Iterate over list of teams.
                    for (int  i = 0; i < 30; i++)
                    {
                        //Find random team in list of all teams.
                        if (randomTeam.getName() == BaseballTeam::teamWAndL[i].getName())
                        {
                            //Replace team in array with randomly created team.
                            //BaseballTeam::teamWAndL[i] = randomTeam;
                            
                            //Point to address of team in array.
                            favoriteTeam = & BaseballTeam::teamWAndL[i];
                            
                            break;
                        }
                    }
                }
                else
                {
                    //Set favorite team.
                    favoriteTeam = specifyTeam(teamIdentifier);
                    
                    //If didn't find team to set as favorite.
                    if (favoriteTeam == NULL)
                    {
                        //Error!
                        errorCount++;
                        callError(errorCount, teamNotFound);
                        
                        break;
                    }
                }
                
                //Dereferenced by -> since pointer to an instance of the class.
                favoriteTeam->setFavoriteTeam(true);
                
                std::cout << "Your team is the " << favoriteTeam->getCity() << " " << favoriteTeam->getName() << "!"<< std::endl << std::endl;
                
                break;
            }
            
            //Start a new season.
            case 'n':
            {
                //If at least part of a season has been played.
                if (seasonsPlayed > 0)
                {
                    //If no games played in current season.
                    if (gamesPlayed == 0)
                    {
                        std::cout << std::endl << "The " << currentSeason << " Major League Baseball season has only just begun!" << std::endl << std::endl;
                        break;
                    }
                    //If less than 162 games played in current season.
                    else if (gamesPlayed < 162)
                    {
                        std::cout << std::endl << "In an early end to the " << currentSeason << " Major League Baseball season, marred by yet another strike, your " << favoriteTeam->getCity() << " " << favoriteTeam->getName();
                        
                        //
                        if ((favoriteTeam->homeWins() + favoriteTeam->awayWins()) > (favoriteTeam->homeLosses() + favoriteTeam->awayLosses()))
                        {
                            std::cout << " were a rare highlight, going ";
                        }
                        else
                        {
                            std::cout << " were just as disappointing, going ";
                        }
                        
                        std::cout << (favoriteTeam->homeWins() + favoriteTeam->awayWins()) << "-" << (favoriteTeam->homeLosses() + favoriteTeam->awayLosses()) << " on the season." << std::endl << std::endl;
                    }
                    else
                    {
                        //I would never say "an historic" anywhere else.
                        std::cout << std::endl << "That concludes an historic " << currentSeason << " season in Major League Baseball, with your " << favoriteTeam->getCity() << " " << favoriteTeam->getName();
                        
                        //Depending on win-loss ratio; dereferenced by -> since pointer to an instance of the class.
                        if ((favoriteTeam->homeWins() + favoriteTeam->awayWins()) > (favoriteTeam->homeLosses() + favoriteTeam->awayLosses()))
                        {
                            std::cout << " hoping to build on a memorable ";
                        }
                        else
                        {
                            std::cout << " looking to improve on a forgettable ";
                        }
                        
                        std::cout << (favoriteTeam->homeWins() + favoriteTeam->awayWins()) << "-" << (favoriteTeam->homeLosses() + favoriteTeam->awayLosses()) << " season." << std::endl << std::endl;
                        
                        std::cout << "Onto " << currentSeason + 1 << "!" << std::endl;
                        
                    }
                    
                    //Increment season.
                    currentSeason++;
                }
                
                //Up count of number of seasons played.
                seasonsPlayed++;
                
                //Reset number of games played in current season.
                gamesPlayed = 0;
                
                //Parse array of teams to reset stats.
                for (int i = 0; i < BaseballTeam::numTeams; i++)
                {
                    BaseballTeam::teamWAndL[i].newSeason(currentSeason);
                }
                
                //If no favorite team chosen.
                if (favoriteTeam == NULL)
                {
                    std::cout << std::endl << "Welcome to the start of the " << currentSeason << " baseball season! Why don't you (c)hoose your favorite team?" << std::endl << std::endl;
                }
                
                break;
            }
            
            //Play a single game.
            case 'p':
            {
                //If more than 162 games played in current season.
                if (gamesPlayed >= 162)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, tooManyGamesPlayed);
                    break;
                }
                
                //If no favorite team selected.
                if (favoriteTeam == NULL)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, favoriteTeamNotDefined);
                    break;
                }
                
                //If first season hasn't started.
                if (seasonsPlayed == 0)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, seasonNotStarted);
                    break;
                }
                
                std::cout << std::endl << "Play ball!" << std::endl;
                
                //Increment number of games played.
                gamesPlayed++;
                
                //Play single game between favorite team and random opponent.
                //playGame(favoriteTeam, & BaseballTeam::teamWAndL[randomOpponent(favoriteTeam)]);
                
                //Ensure other teams also play a game to keep records even, and always display results of favorite team.
                allTeamsPlay(true);
                
                break;
            }
                
            //Manual entry of wins and losses.
            case 'm':
            {
                //If more than 162 games played in current season.
                if (gamesPlayed >= 162)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, tooManyGamesPlayed);
                    break;
                }
                
                //If no favorite team selected.
                if (favoriteTeam == NULL)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, favoriteTeamNotDefined);
                    break;
                }
                
                //If first season hasn't started.
                if (seasonsPlayed == 0)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, seasonNotStarted);
                    break;
                }
                
                std::cout << std::endl << "Enter (w)in/s or (l)oss/es? ";
                
                char choiceWOrL;
                std::cin >> choiceWOrL;
                
                //Flush buffer. A bit more verbose than fflush from C.
                std::cin.clear();
                std::cin.ignore(INT_MAX, '\n');
                
                std::cout << "Enter (h)ome or (a)way? ";
                
                char choiceHOrA;
                std::cin >> choiceHOrA;
                
                //Flush buffer. A bit more verbose than fflush from C.
                std::cin.clear();
                std::cin.ignore(INT_MAX, '\n');
                
                std::cout << "Enter number of games: ";
                
                int numGames;
                std::cin >> numGames;
                
                //Flush buffer. A bit more verbose than fflush from C.
                std::cin.clear();
                std::cin.ignore(INT_MAX, '\n');
                
                //If more than 162 games played in current season.
                if (numGames + gamesPlayed > 162)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, tooManyGamesPlayed);
                    break;
                }
                
                //If erroneous input.
                if (choiceWOrL != 'w' && choiceWOrL != 'W' && choiceWOrL != 'l' && choiceWOrL != 'L')
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, invalidSelection);
                    break;
                }
                
                //If erroneous input.
                if (choiceHOrA != 'h' && choiceHOrA != 'H' && choiceHOrA != 'a' && choiceHOrA != 'A')
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, invalidSelection);
                    break;
                }
                
                //If negative games played.
                if (numGames < 0)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, cantUnplayGame);
                    break;
                }
                else
                {
                    //Ensure all other teams play same number of games as entered manually.
                    for (int i = 0; i < numGames; i++)
                    {
                        allTeamsPlay(false, true, choiceHOrA == 'h' || choiceHOrA == 'H', choiceWOrL == 'w' || choiceWOrL == 'W');
                    }
                    
                    //Increment number of games played this season by the number of manually entered games.
                    gamesPlayed += numGames;
                }
                
                break;
            }
                
            //Simulate season.
            case 's':
            {
                //If more than 162 games played in current season.
                if (gamesPlayed >= 162)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, tooManyGamesPlayed);
                    break;
                }
                
                //If no favorite team selected!
                if (favoriteTeam == NULL)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, favoriteTeamNotDefined);
                    break;
                }
                
                //If first season hasn't started.
                if (seasonsPlayed == 0)
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, seasonNotStarted);
                    break;
                }
                
                std::cout << std::endl << "Show all results? (y)es or (n)o? ";
                
                //Declare character variable to handle choice input.
                char choiceDisplay;
                std::cin >> choiceDisplay;
                
                //Flush buffer. A bit more verbose than fflush from C.
                std::cin.clear();
                std::cin.ignore(INT_MAX, '\n');
                
                //Declare boolean variable to handle whether results should be displayed.
                bool displayResults;
                
                //Display results if yes, don't display if no.
                if (choiceDisplay == 'y' || choiceDisplay == 'Y')
                {
                    displayResults = true;
                }
                else if (choiceDisplay == 'n' || choiceDisplay == 'N')
                {
                    displayResults = false;
                }
                else
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, invalidSelection);
                    break;
                }

                //Simulate rest of games necessary to play full season.
                simulateSeason(162 - gamesPlayed, displayResults);
                
                //Set number of games played this season to 162 since all games have been played.
                gamesPlayed = 162;
                
                break;
            }
            
            //Display team stats.
            case 'd':
            {
                //If favorite team already chosen.
                if (favoriteTeam != NULL)
                {
                    std::cout << std::endl << "Would you like to display stats for your (f)avorite team, or a (s)pecified team?" << std::endl;
                    
                    //Declare character variable to handle choice input.
                    char choiceFOrS;
                    std::cin >> choiceFOrS;
                    
                    //Flush buffer. A bit more verbose than fflush from C.
                    std::cin.clear();
                    std::cin.ignore(INT_MAX, '\n');
                    
                    //If opted to display stats for favorite team.
                    if (choiceFOrS == 'f' || choiceFOrS == 'F')
                    {
                        //Print stats for favorite team, dereferenced by ->.
                        favoriteTeam->printTeam();
                    }
                    //If opted to display stats for another team.
                    else if (choiceFOrS == 's' || choiceFOrS == 'S')
                    {
                        std::cout << std::endl << "For which team do you want to display stats? Start by entering the city or team name." << std::endl;
                        std::string teamIdentifier = "";
                        std::getline(std::cin, teamIdentifier);
                        
                        //Ensure valid team by checking against list of teams.
                        BaseballTeam * specifiedTeam = specifyTeam(teamIdentifier);
                        
                        //If no team exists.
                        if (specifiedTeam == NULL)
                        {
                            //Error!
                            errorCount++;
                            callError(errorCount, teamNotFound);
                            
                            break;
                        }
                        //If team exists.
                        else
                        {
                            //Print stats for team, dereferenced by ->.
                            specifiedTeam->printTeam();
                        }
                    }
                    //If invalid input.
                    else
                    {
                        //Error!
                        errorCount++;
                        callError(errorCount, invalidSelection);
                    }
                }
                //If favorite team not chosen.
                else
                {
                    //Error!
                    errorCount++;
                    callError(errorCount, favoriteTeamNotDefined);
                }
                
                break;
            }
            
            //Quit.
            case 'q':
            {
                //Exit program.
                std::cout << std::endl << "You're out!" << std::endl;
                exit(4);
            }
            
            //Erroneous input.
            default:
            {
                //Error!
                errorCount++;
                callError(errorCount, invalidSelection);
            }
        }
    //Repeat infinitely.
    } while (repeater);

    return 0;
}

//
//Function: bool caseInsensitiveStringCompare(const std::string str1, const std::string str2)
//Purpose: To check if the strings are equal, disregarding case.
//Parameters: The parameters str1 and str2 can be any strings.
//Return: True if the strings are equal, false otherwise.
//
bool caseInsensitiveStringCompare(const std::string str1, const std::string str2)
{
    if (str1.size() != str2.size())
    {
        return false;
    }
    //for (std::string::const_iterator c1 = str1.begin(), c2 = str2.begin(); c1 != str1.end(); ++c1, ++c2)
    
    for (int i = 0; i < str1.size(); i++)
    {
        if (tolower(str1[i]) != tolower(str2[i]))
        {
            return false;
        }
    }
    return true;
}

//
//Function: BaseballTeam * specifyTeam(std::string teamIdentifier)
//Purpose: To check team specified by the user in main() against the list of all teams.
//Parameters: The parameter teamIdentifier is a copy from main() that gets checked against the list of all teams.
//Return: The return value specifiedTeam is a pointer to an object of type BaseballTeam.
//Input: If user indicated Los Angeles or New York as favorite team in main(), asks for clarification.
//Output: Prompts to differentiate between the two teams in LA and NY, respectively, and also lets the user know if a match was found.
//
BaseballTeam * specifyTeam(std::string teamIdentifier)
{
    BaseballTeam * specifiedTeam = NULL;
    
    //'Cause LA is special.
    if (caseInsensitiveStringCompare(teamIdentifier, "Los Angeles"))
    {
        std::cout << "Did you mean the Los Angeles Dodgers or the Los Angeles Angels? Please enter team name only." << std::endl;
        
        std::string teamDifferentiator;
        std::cin >> teamDifferentiator;
        
        //Flush buffer. A bit more verbose than fflush from C.
        std::cin.clear();
        std::cin.ignore(INT_MAX, '\n');
        
        //If a Dodgers fan.
        if (caseInsensitiveStringCompare(teamDifferentiator, "Dodgers"))
        {
            //Search by team name.
            teamIdentifier = teamDifferentiator;
        }
        //If an Angels fan.
        else if (caseInsensitiveStringCompare(teamDifferentiator, "Angels"))
        {
            //Search by team name.
            teamIdentifier = teamDifferentiator;
        }
        //If messing with the system.
        else
        {
            return NULL;
        }
    }
    //I reckon NY is too.
    else if (caseInsensitiveStringCompare(teamIdentifier, "New York"))
    {
        std::cout << "Did you mean the New York Mets or the New York Yankees? Please enter team name only." << std::endl;
        
        std::string teamDifferentiator;
        std::cin >> teamDifferentiator;
        
        //Flush buffer. A bit more verbose than fflush from C.
        std::cin.clear();
        std::cin.ignore(INT_MAX, '\n');
        
        //If a Mets fan.
        if (caseInsensitiveStringCompare(teamDifferentiator, "Mets"))
        {
            //Search by team name.
            teamIdentifier = teamDifferentiator;
        }
        //If a Yankees fan.
        else if (caseInsensitiveStringCompare(teamDifferentiator, "Yankees"))
        {
            //Search by team name.
            teamIdentifier = teamDifferentiator;
        }
        //If messing with the system.
        else
        {
            return NULL;
        }
    }
    
    //Iterate over list of teams.
    for (int  i = 0; i < 30; i++)
    {
        //If found.
        if (caseInsensitiveStringCompare(teamIdentifier, BaseballTeam::teamWAndL[i].getCity()) || caseInsensitiveStringCompare(teamIdentifier, BaseballTeam::teamWAndL[i].getName()))
        {
            std::cout << "I found a matching team!" << std::endl << std::endl;
            
            //Point to address of team in array.
            specifiedTeam = & BaseballTeam::teamWAndL[i];
            
            break;
        }
    }
    
    return specifiedTeam;
}

//
//Function: int randomOpponent(BaseballTeam * favoriteTeam)
//Purpose: To select random opponent in order to play a game.
//Parameters: The parameter BaseballTeam * favoriteTeam is a pointer to the specified favorite team, used to determine matchup.
//Return: The return value int index is the index of the randomly selected opponent.
//Input: N/A
//Output: N/A
//
int randomOpponent(BaseballTeam * favoriteTeam)
{
    //Declare integer variable to handle index lookup.
    int index;
    
    //As long as...
    do {
        //Select random index from 0-29.
        index = random() % 30;
        
    //...favorite team is the team found at the index.
    } while(favoriteTeam == & BaseballTeam::teamWAndL[index]);

    return index;
}

//
//Function: void playGame(BaseballTeam * teamA, BaseballTeam * teamB, bool displayResults)
//Purpose: To play an individual game, determining home and away team, outcome with appropriate home bias and game score; updating records of home wins and losses and away wins and losses; and displaying details of the game.
//Parameters: The parameters BaseballTeam * teamA and BaseballTeam * teamB are pointers to teams passed from allTeamsPlay, while displayResults is a boolean variable that handles whether or not to display game details.
//Return: N/A
//Input: N/A
//Output: Reports outcome of the game including opponent and score.
//
void playGame(BaseballTeam * teamA, BaseballTeam * teamB, bool displayResults)
{
    //Declare pointers to home and away teams for assignment.
    BaseballTeam * homeTeam = NULL;
    BaseballTeam * awayTeam = NULL;
    
    //Randomly decide which teams are home and away.
    if (random() % 2)
    {
        homeTeam = teamA;
        awayTeam = teamB;
    }
    else
    {
        homeTeam = teamB;
        awayTeam = teamA;
    }
    
    //Declare a boolean variable to handle tracking of whether home team won.
    bool homeWon;
    
    //Bias towards home team winning, away team losing, with remainders of 0 or 1 signifying loss and remainders of 2, 3 or 4 signifying win.
    if ((random() % 5) > 1)
    {
        //Home team wins!
        homeTeam->win(1, true);
        homeWon = true;
        
        //Away team loses!
        awayTeam->lose(1, false);
    }
    else
    {
        //Home team loses!
        homeTeam->lose(1, true);
        homeWon = false;
        
        //Away team wins!
        awayTeam->win(1, false);
    }
    
    //If a game in which favorite team is playing.
    if (homeTeam->isFavoriteTeam() || awayTeam->isFavoriteTeam())
    {
        //Declare integer variables to handle random selection of score.
        int score1, score2;
        
        //Assign random number to first score.
        score1 = random() % 8;
        
        //As long as...
        do{
            //Assign random number to second score.
            score2 = score1 + random() % 8;
        //...first score is greater than or equal to second score.
        } while(score1 >= score2);
        
        //If results to be displayed.
        if (displayResults)
        {
            //If home team won...
            if (homeWon)
            {
                //...and it's the user's favorite team.
                if (homeTeam->isFavoriteTeam())
                {
                    std::cout << "Your " << homeTeam->getCity() << " " << homeTeam->getName() << " defeated the " << awayTeam->getCity() << " " << awayTeam->getName() << " by a " << score2 << "-" << score1 << " margin!" << std::endl;
                }
                //...and it's the opponent.
                else
                {
                    std::cout << "Your " << awayTeam->getCity() << " " << awayTeam->getName() << " fell on the road to the " << homeTeam->getCity() << " " << homeTeam->getName() << ", " << score2 << "-" << score1 << "!" << std::endl;
                }
            }
            //If away team won...
            else
            {
                //...and it's the opponent.
                if (homeTeam->isFavoriteTeam())
                {
                    std::cout << "Your " << homeTeam->getCity() << " " << homeTeam->getName() << " sufferred a home loss to the " << awayTeam->getCity() << " " << awayTeam->getName() << " by a score of " << score2 << "-" << score1 << "!" << std::endl;
                }
                //...and it's the user's favorite team.
                else
                {
                    std::cout << "Your " << awayTeam->getCity() << " " << awayTeam->getName() << " took down the " << homeTeam->getCity() << " " << homeTeam->getName() << ", " << score2 << "-" << score1 << ", on the road!" << std::endl;
                }
            }
        }
    }
    
    //No return!
}

//
//Function: playRiggedGame(BaseballTeam * teamA, BaseballTeam * teamB, bool favoriteHome, bool favoriteWon)
//Purpose: To play a rigged game, favorite team v random team, to justify manual input of games won or lost.
//Parameters: The parameters BaseballTeam * teamA and BaseballTeam * teamB are pointers to teams passed from allTeamsPlay, while favoriteHome and favoriteWon are booleans variables that handle whether the user's favorite team was the home team and the winner, respectively.
//Return: N/A
//Input: N/A
//Output: N/A
//
void playRiggedGame(BaseballTeam * teamA, BaseballTeam * teamB, bool favoriteHome, bool favoriteWon)
{
    //Declare pointers to favorite and opponent teams for assignment.
    BaseballTeam * favoriteTeam = NULL;
    BaseballTeam * opponentTeam = NULL;
    
    //Randomly decide which teams are home and away.
    if (teamA->isFavoriteTeam())
    {
        favoriteTeam = teamA;
        opponentTeam = teamB;
    }
    else
    {
        favoriteTeam = teamB;
        opponentTeam = teamA;
    }
    
    //If favorite team won, add wins and losses to both teams' records as appropriate.
    if (favoriteWon)
    {
        favoriteTeam->win(1, favoriteHome);
        
        opponentTeam->lose(1, !favoriteHome);
    }
    //If favorite team lost, add wins and losses to both teams' records as appropriate.
    else
    {
        favoriteTeam->lose(1, favoriteHome);
        
        opponentTeam->win(1, !favoriteHome);
    }
    
    //No return!
}

//
//Function: void allTeamsPlay(bool displayResults, bool rigGame, bool favoriteHome, bool favoriteWon)
//Purpose: To ensure all teams play equal amount of games.
//Parameters: The parameter displayResults is a boolean variable that specifies whether game details are to be displayed and rigGame is a boolean variable that specifies whether the game is the result of a random game or manual user input, while favoriteHome and favoriteWon are booleans variables that handle whether the user's favorite team was the home team and the winner, respectively.
//Return: N/A
//Input: N/A
//Output: N/A
//
void allTeamsPlay(bool displayResults, bool rigGame, bool favoriteHome, bool favoriteWon)
{
    //Declare array of indices associated with teams.
    int indices[30];
    
    //Divide size of entire array by size of individual element to calculate number of elements in array.
    int indexCount = sizeof(indices)/sizeof(indices[0]);
    
    //Ensuring array of teams has one of each index.
    for (int i = 0; i < indexCount; i++)
    {
        //Index 0 = 0...index 29 = 29.
        indices[i] = i;
    }
    
    //Shuffle the list of indices using a modern version of the Fisher-Yates shuffle algorithm. About Fisher-Yates shuffle algorithm: https://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle#The_modern_algorithm
    
    //Start with the rightmost index.
    for (indexCount--; indexCount >= 1; indexCount--)
    {
        //Select a random index from 0 to indexCount, inclusive.
        int randomIndex = random() % (indexCount + 1);
        
        //Swap the two indices.
        int indexA = indices[indexCount];
        int indexB = indices[randomIndex];
        indices[indexCount]  = indexB;
        indices[randomIndex] = indexA;
    }
    
    //Reset index count.
    indexCount = sizeof(indices)/sizeof(indices[0]);
    
    //Parse array of teams.
    for (int i = 0; i < indexCount; i += 2)
    {
        //Pick up two consecutive teams in shuffled array.
        BaseballTeam * teamA = &BaseballTeam::teamWAndL[indices[i]];
        BaseballTeam * teamB = &BaseballTeam::teamWAndL[indices[i + 1]];
        
        //If game is rigged--that is, manually entered--call function that uses predetermined outcome.
        if (rigGame && (teamA->isFavoriteTeam() || teamB->isFavoriteTeam()))
        {
            playRiggedGame(teamA, teamB, favoriteHome, favoriteWon);
        }
        //If game is to be randomized, simulate game played between two given teams, passing pointers to teams at chosen array indices.
        else
        {
            playGame(teamA, teamB, displayResults);
        }
    }
    
    //No return!
}

//
//Function: void simulateSeason(int gamesToPlay, bool displayResults)
//Purpose: To simulate an entire season or the remainder of a season in which games have already been played randomly or entered manually.
//Parameters: The parameter gamesToPlay is an integer variable that indicates how many games are to be played and displayResults is a boolean variable that specifies whether game details are to be displayed.
//Return: N/A
//Input: N/A
//Output: N/A
//
void simulateSeason(int gamesToPlay, bool displayResults)
{
    //For duration of season, remainder of 162 games.
    for (int i = 0; i < gamesToPlay; i++)
    {
        //Ensure each team plays 162 games.
        allTeamsPlay(displayResults);
    }
    
    if (displayResults)
    {
        std::cout << std::endl;
    }
    
    //No return!
}

//
//Function: void callError(int errorCount, ERROR_TYPE errorType)
//Purpose: To handle error output.
//Parameters: The parameter errorCount is an integer variable that stores a cumulative count of errors encountered in the program, and errorType is an enumerated type that corresponds to the error output.
//Return: N/A
//Input: N/A
//Output: Displays a modified count of accumulated errors in the program interactions as well as notifications to the user explaining the error encountered.
//
void callError(int errorCount, ERROR_TYPE errorType)
{
    //Creatively handling erroneous input without throwing and catching exceptions.
    if (errorCount == 1)
    {
        std::cout << std::endl << "A swing and a miss! Strike one!" << std::endl << std::endl;
    }
    if (errorCount == 2)
    {
        std::cout << std::endl << "Caught looking! Strike two!" << std::endl << std::endl;
    }
    if (errorCount == 3)
    {
        std::cout << std::endl << "The grounder bounces into foul territory! Foul ball!" << std::endl << std::endl;
    }
    if (errorCount == 4)
    {
        std::cout << std::endl << "Tipped back behind home plate! Foul ball!" << std::endl << std::endl;
    }
    if (errorCount == 5)
    {
        std::cout << std::endl << "Strike three! You're outta here!" << std::endl << std::endl;
        exit(3);
    }
    
    switch (errorType) {
        case teamNotFound:
        {
            std::cout << "Is that some AAA affiliate? No such team exists in Major League Baseball!" << std::endl << std::endl;
            break;
        }
            
        case favoriteTeamNotDefined:
        {
            std::cout << "And you call yourself a fan? You haven't selected a favorite team! Choose your team, will ya?" << std::endl << std::endl;
            break;
        }
            
        case tooManyGamesPlayed:
        {
            std::cout << "The Major League Baseball season is 162 games, barring a strike! You can't play that many games!" << std::endl << std::endl;
            break;
        }
            
        case seasonNotStarted:
        {
            std::cout << "The season hasn't yet begun. Go on and start a new season, will ya?" << std::endl << std::endl;
            break;
        }
            
        case invalidSelection:
        {
            std::cout << "Losing is not an option, and neither is the choice you entered!" << std::endl << std::endl;
            break;
        }
            
        case cantUnplayGame:
        {
            std::cout << "You can't unplay a game! Try that again, will ya?" << std::endl << std::endl;
            break;
        }
    }
    
    //No return!
}




/*

 BaseballTeam la( "Dodgers",
 "Los Angeles" );
 la.printTeam( );  // The NL team: Los Angeles Dodgers has a 0-0 record
 
 la.win( );
 la.lose( );
 la.printTeam( );  // The NL team: Los Angeles Dodgers has a 1-1 record
 
 la.newSeason( 2014 ); //  The NL team: Los Angeles Dodgers - 2014 season starting
 
 la.win( 45, true );  // 45 home wins
 la.lose( 36, true ); // 36 home loss
 la.win( 49, false ); // 49 away wins
 la.lose( 32, false );// 32 away loss
 
 la.printTeam( );  // The NL team: Los Angeles Dodgers has a 94-68 record
 
 // now some errors...
 la.newSeason( -100 );  // The NL team: Los Angeles Dodgers can't have a -100 season!
 
 la.newSeason( 2015 );  // The NL team: Los Angeles Dodgers - 2015 season starting
 la.win( 1000, true );  // he NL team: Los Angeles Dodgers - Season is only 162 games!
 
 la.newSeason( 2015 );  // The NL team: Los Angeles Dodgers - 2015 season starting
 la.lose( 1000, false ); // The NL team: Los Angeles Dodgers - Season is only 162 games!
 
 la.newSeason( 2015 ); // The NL team: Los Angeles Dodgers - 2015 season starting
 la.lose( -10 ); // The NL team: Los Angeles Dodgers - count can't be negative!
 
 la.newSeason( 2015 ); // The NL team: Los Angeles Dodgers - 2015 season starting
 la.win( -10 ); // The NL team: Los Angeles Dodgers - count can't be negative!

*/



/*
      F F F F    A A    N     N  T T T T    A A     S S S   Y     Y
      F        A     A  N N   N     T     A     A  S        Y     Y
      F F F    A A A A  N   N N     T     A A A A   S S S    Y Y Y
      F        A     A  N     N     T     A     A        S        Y
      F        A     A  N     N     T     A     A   S S S    Y Y Y
 
 B B B      A A     S S S   E E E E  B B B      A A    L        L
 B    B   A     A  S        E        B    B   A     A  L        L
 B B B    A A A A   S S S   E E E    B B B    A A A A  L        L
 B     B  A     A        S  E        B     B  A     A  L        L
 B B B    A     A   S S S   E E E E  B B B    A     A  L L L L  L L L L
 */
